#!/bin/bash
#
# pgn.sh
#
# Converts PGN data to HTML via template per pgn-viewer requirements.  
# HTML rendering of PGN may be played back and/or edited.
#
# Wrapper around pgn-viewer.  SEE https://github.com/mliebelt/pgn-viewer.
#
#
# Separates PGN core data from any notes above or below by matching on
#  . PGN Tab Pair(s) strings (eg: [ABC "XYZ"]), and/or
#  . game results string at end of Movetext (eg: {0-1}).  
#
#       ** AVOID THESE TRIGGER STRINGS in notes. **
#
# PGN notes may include HTML markup.
#
# Performs a few other tasks to understand pgn-viewer as service 
# and chess reference.
#
#
#
# SETUP--
#   . Download and build pgn-viewer -- https://github.com/mliebelt/pgn-viewer.
#   . To enable this script, customize lines below commented with "INSTALL".
#   . Place this script and associated assets (pgn-template.html,
#       nags-demo.html) together in same directory recognized by
#       PATH environment variable.
#
# TBD--
#   . Name HTMLPRODUCT per PGN TagPairs?  Disambiguate via date and/or index.
#
#
#---------------------------------------------------------------------
#     Copyright (C) David Reeder 2024.  chess@mobilesound.org
#     Distributed under the Boost Software License, Version 1.0.
#     (See ./LICENSE_1_0.txt or http://www.boost.org/LICENSE_1_0.txt.)
#---------------------------------------------------------------------
#

USAGE="Usage: $(basename $0) [-hH] [-dinpt] pgnFile [subDir]"

USAGE_LONG="
  -d        Open directory where HTML results are stored.
  -i        Open pgn-viewer documentation in browser.
  -n        Demo of Numeric Annotation Glyphs (NAGs) presented by pgn-viewer. 
  -p        Post path to directory of HTML results.
  -t        Open HTML in browser tab, instead of new window.  (When supported.)

  -h/-H     Usage (long).

  pgnFile   Path to PGN file to convert to HTML.
  subDir    Store HTML result in <subdir> under root directory of HTML results.
  "

VERSION="0.1"   #RELEASE




#--------------------------------------- -o-
# Globals.

CHESSHOME="$HOME/project/chess/"                        #INSTALL
PGNROOT="$CHESSHOME/project/games/"                     #INSTALL
PGNDIR="$PGNROOT/pgnv/"                                 #INSTALL

PGNFILE=
SUBDIR=
HTMLPRODUCT=

PGNVHOME="$CHESSHOME/project/pgnviewer/pgn-viewer/"     #INSTALL
PGNVDOCS="$PGNVHOME/docs/index.html"

PGNTEMPLATE="$(dirname $0)/$(basename $0 .sh)-template.html"
NAGSDEMO="$(dirname $0)/nags-demo.html"

FIREFOXAPP="/Applications/Firefox.app/Contents/MacOS/firefox";
                                                        #INSTALL
FIREFOXARGS=


#DEBUGGING=true   #NB  Enabled when DEBUGGING has length greater than zero.


#
RWNOTES="_REPLACE_WITH_NOTES_"
RWHEADER="_REPLACE_WITH_HEADER_"
RWPGN_ESCAPED="_REPLACE_WITH_PGN_ESCAPED_"
RWPGN_ORIGINAL="_REPLACE_WITH_PGN_ORIGINAL_"
RWTITLE="_REPLACE_WITH_TITLE_"

DOT=.
[ "$DEBUGGING" ] && DOT=
TMPFILEBASE="$ht/$DOT$(basename $0)-tmp$$-"

TMPNOTES="${TMPFILEBASE}notes.txt"
TMPNOTES_MARKUP="${TMPFILEBASE}notes_markup.txt"
TMPHEADER="${TMPFILEBASE}header.txt" 
TMPPGN_ESCAPED="${TMPFILEBASE}pgn_escaped.txt"
TMPPGN_ORIGINAL="${TMPFILEBASE}pgn_original.txt" 
TMPPGN_TAGPAIRS="${TMPFILEBASE}tagpairs.txt" 
TMPTITLE="${TMPFILEBASE}title.txt" 
TMPTITLE_SHORTER="${TMPFILEBASE}title_shorter.txt" 

FMTWIDTH="75"




#--------------------------------------- -o-
# Functions.

postUsage()
{
  local  LONG=$1

  echo $USAGE  1>&2

  [ "$LONG" = long ] && {
    cat <<LONGUSAGE  1>&2
$USAGE_LONG  

  PGNDIR is $PGNDIR

LONGUSAGE
  }

  exit  1
}



# Separate PGN from notes above and below,
#   via match on Tag Pairs ("[Event ...]", starting a line) 
#     or first move ("1.", starting a line)
#   ended by Movelist terminator/game result (ending a line).
# Quote PGN for use in HTML file.
# Source Tag Pairs as envirnoment variables.
# Generate game title and header.
#
minePGNFile()  # pgnFile
{
  local  PGNFILE=$1
  local  CONNECTOR=
  local  EVENTSITEDATEROUND=
  local  TITLE=
  local  RESULT=


  # Separate PGN via Tag Pairs or Movetext from notes before and after.
  #
  # captureMode 0 isolates PGN data.  Other modes isolate notes
  #   before (1) or after (2) the PGN data.  
  # ASSUME game data is contiguous.  All other lines qualify as notes.
  #
  cat $PGNFILE | 
  awk '
    BEGIN { captureMode = 1     
            if (length("'$DEBUGGING'") > 0) { print "DEBUG awk captureMode=1" }
          }

    length("'$DEBUGGING'") > 0  { printf "DEBUG --%s\n", $0 }

    /^[ \t]*\[.+[ \t]+\".*\"[ \t]*\]/  ||       # Tag Pairs, or...
    /^[ \t]*1\./                       {        # ...begin Movetext...
                            if (captureMode >= 2) {
                              print  >"'$TMPNOTES'"
                              next
                            }
                            captureMode = 0
                            if (length("'$DEBUGGING'") > 0) { 
                              print "DEBUG awk captureMode=0" 
                            }
                          }                                   # ...end Movetext.
                                             /\*[ \t]*$/  ||  # *
                                       /{[ \t]*}[ \t]*$/  ||  # {}
                               /{[ \t]*\*[ \t]*}[ \t]*$/  ||  # {*}
    /{[ \t]*[012\/]+[ \t]*-[ \t]*[012\/]+[ \t]*}[ \t]*$/  ||  # {N-N}
            /[012\/]+[ \t]*-[ \t]*[012\/]+[ \t]*[ \t]*$/  {   # N-N
                          if (length("'$DEBUGGING'") > 0) { 
                            print "DEBUG awk captureMode=2" 
                          }
                          captureMode = 2
                          print  >"'$TMPPGN_ORIGINAL'"
                          next 
                        }
    captureMode == 0    { print  >"'$TMPPGN_ORIGINAL'" }
    captureMode >  0    { print  >"'$TMPNOTES'" }
  '

  cat $TMPPGN_ORIGINAL | sed -e 's,$, \\,' -e "s,',\\\',g"  >$TMPPGN_ESCAPED

  touch  $TMPNOTES      # Prevent errs when file is not used.


  # Vertical spacing for notes without HTML.
  #
  echo $(cat $TMPNOTES) | egrep '<.*>'  1>/dev/null 2>/dev/null
  [ $? -ne 0 ] && {
    {
      echo "<PRE>"
      cat $TMPNOTES  2>/dev/null
      echo "</PRE>"
    }  >$TMPNOTES_MARKUP

    cp -f $TMPNOTES_MARKUP $TMPNOTES  2>/dev/null
  }


  # Source Tag Pairs as environment variables.
  #
  echo $(cat $TMPPGN_ORIGINAL) |
  awk '
    { 
       numTagPairs = split($0, tagPairs, "[")
       count=2

       while (count <= numTagPairs) { printf "TP_%s\n", tagPairs[count++] }

       exit  0
    }
  ' |  
  sed -e 's/\].*//' -e 's/ /=/'  >$TMPPGN_TAGPAIRS

  . $TMPPGN_TAGPAIRS
   

  # Generate game title.
  #
  {
    echo -n "${TP_White:-White}"                         
    [ "$TP_WhiteElo" ] && echo -n "($TP_WhiteElo)"       
    echo -n " "

    echo -n "v. ${TP_Black:-Black}"                     
    [ "$TP_BlackElo" ] && echo -n "($TP_BlackElo)"   
    echo -n " "

    [ "${TP_Result}" ] && {
      echo -n ":: ${TP_Result} "    
    }

    EVENTSITEDATEROUND="$TP_Event"                                   
    CONNECTOR="At"

    [ "$TP_Site" ] && {
      [ "$TP_Event" ] && CONNECTOR=" at"
      EVENTSITEDATEROUND="${EVENTSITEDATEROUND}$CONNECTOR $TP_Site "                      
    }
    CONNECTOR="On"

    [ "$TP_Date" ] && {
      [ "${TP_Event}${TP_Site}" ] && CONNECTOR="on"
      EVENTSITEDATEROUND="${EVENTSITEDATEROUND}$CONNECTOR $TP_Date "
    }

    [ "$TP_Round" ] && {
      EVENTSITEDATEROUND="${EVENTSITEDATEROUND}(Round $TP_Round)"
    }

    [ "$EVENTSITEDATEROUND" ] && {
      echo -n "-- $EVENTSITEDATEROUND"
    }
  }  >$TMPTITLE


  # Generate game header.
  #
  {
    cat $TMPTITLE | sed -e 's/::.*$//' -e 's/--.*$//'

    [ "${TP_Result}${TP_Termination}" ] && {
      CONNECTOR=
      [ "$TP_Result" -a "$TP_Termination" ] && CONNECTOR=" :: "
        
      RESULT="${TP_Result}${CONNECTOR}${TP_Termination}"    
    }
    
    cat $TMPTITLE | egrep '::|--'   >/dev/null  2>/dev/null
    [ $? -eq 0 ] && TITLE="$(cat $TMPTITLE | sed -e 's/^.*:://' -e 's/^.*--//')"

    [ "${RESULT}${TITLE}${TP_Annotator}" ] && {
      echo "<BLOCKQUOTE>"

      [ "$RESULT" ] && echo "$RESULT" 

      [ "$TITLE" ] && {
        [ "$RESULT" ] && echo "<BR>"
        echo $TITLE
      }

      [ "$TP_Annotator" ] && {
        [ "$TITLE$RESULT" ] && echo "<BR><BR>"
        echo "Annotator ${TP_Annotator}"
      }

      echo "</BLOCKQUOTE>"
    }
  }  >$TMPHEADER

  cat $TMPTITLE | sed 's,([0-9]*),,g'  >$TMPTITLE_SHORTER


  #
  [ "$DEBUGGING" ] && {
    echo
    echo DEBUG ___________________________________
    echo DEBUG -- TITLE
    cat $TMPTITLE

    echo; echo; echo DEBUG -- HEADER
    cat $TMPHEADER

    echo; echo DEBUG -- PGN
    cat $TMPPGN_ORIGINAL

    echo; echo DEBUG -- NOTES
    cat $TMPNOTES  
    echo DEBUG ___________________________________
    echo
  }
}


renderPGNFile()  # pgnFile
{
  local  PGNFILE=$1
  local  HTMLFILENAME=$(basename $PGNFILE .pgn).html
  local  ABBREVIATED_HTMLFILENAME=


  #
  minePGNFile $PGNFILE

  [ "$SUBDIR" ] && {
    mkdir -p $PGNDIR/$SUBDIR
    rm -f $PGNDIR/$SUBDIR/.PGNVHOME
    ln -s $PGNDIR/.PGNVHOME $PGNDIR/$SUBDIR/  2>/dev/null
  }

  HTMLPRODUCT=$PGNDIR/$SUBDIR/$HTMLFILENAME

  [ -e "$HTMLPRODUCT" ] && {
    ABBREVIATED_HTMLFILENAME=$(echo $HTMLPRODUCT | 
      sed "s,^$(dirname $PGNDIR),...," | sed -e 's,//,/,g' -e 's,//,/,g')


    echo
    echo "Remove and update existing PGN file \"$ABBREVIATED_HTMLFILENAME\"?"

    echo; echo -n "    "
    rm -i "$HTMLPRODUCT" 
    echo
    [ -e "$HTMLPRODUCT" ] && exit 1
  }


  cat $PGNTEMPLATE | 
  awk '
    /'$RWNOTES'/        { system("cat '$TMPNOTES'");            next; } 
    /'$RWHEADER'/       { system("cat '$TMPHEADER'");           next; } 
    /'$RWPGN_ESCAPED'/  { system("cat '$TMPPGN_ESCAPED'");      next; } 
    /'$RWPGN_ORIGINAL'/ { 
        system("cat '$TMPPGN_ORIGINAL' | fmt -'$FMTWIDTH'");    next; } 
    /'$RWTITLE'/        { system("cat '$TMPTITLE_SHORTER'");    next; } 
    { print }
  '  >$HTMLPRODUCT  


  #
  [ -z "$DEBUGGING" ] && rm -f ${TMPFILEBASE}*  
}




#--------------------------------------- -o-
# Main.

[ $# -eq 0 ] && postUsage

while [ "$1" ]; do
  case "$1" in
  -d|d)         open $PGNDIR                    ;;
  -i|i)         open $PGNVDOCS                  ;;
  -n|n)         open $NAGSDEMO                  ;;
  -p|p)         echo $PGNDIR                    ;;
  -t|t)         FIREFOXARGS="--new-tab"         ;;      #INSTALL

  -h|h)         postUsage                       ;;
  -H|H)         postUsage long                  ;;

  *)            [ "$PGNFILE" ] && { 
                  [ "$SUBDIR" ] && postUsage
                  SUBDIR=$1
                } || {
                  PGNFILE=$1
                }
                                                ;;
  esac

  shift
done


[ -z "$PGNFILE" ] && exit 0

[ ! -e "$PGNFILE" ] && {
  echo "$(basename $0): Cannot find file \"$PGNFILE\"."  1>&2
  postUsage
}


#
mkdir -p $(dirname $TMPFILEBASE)

[ ! -e "$PGNDIR" ] && mkdir -p $PGNDIR

rm -f $PGNDIR/.PGNVHOME
ln -s $PGNVHOME $PGNDIR/.PGNVHOME  2>/dev/null


#
renderPGNFile $PGNFILE

[ "$FIREFOXAPP" ] && {
  $FIREFOXAPP $FIREFOXARGS $HTMLPRODUCT
} || {
  open $HTMLPRODUCT        
}


#
exit  0

